@@ -8,6 +8,7 @@
 typedef struct {Long x[AMBI_Dmax][AMBI_Dmax]; int n, N;}     CWLatticeBasis;
 
 void Make_CWS_Points(CWS *_C, PolyPointList *_P);
+void Make_RGC_Points(CWS *Cin, PolyPointList *_P);
 void CWS_to_PermCWS(CWS *Cin, CWS *C, int *pi);
 
 /*  ==========  	  I/O functions:                	==========  */
@@ -99,15 +100,37 @@ int  Read_CWS_Zinfo(FILE *inFILE,CWS *CW)		      /* return !EOF */
      }	return 1;
 }
 
-int  Read_CWS_PP(CWS *_CW, PolyPointList *_P)
+void checkDimension(int polyDim, int codim, int index){
+  if (index == 1){ 
+    if (POLY_Dmax  < (polyDim + codim - 1)){
+      if (codim > 1){
+	printf("Please increase POLY_Dmax to at least %d = %d + %d - 1\n",
+	       (polyDim + codim - 1), polyDim, codim);
+	puts("(nef.x requires POLY_Dmax >= dim N + codim - 1)");}
+      else printf("Please increase POLY_Dmax to at least %d\n", polyDim);
+      exit(0);  }}
+  else if (POLY_Dmax  < polyDim +1){
+    printf("Please increase POLY_Dmax to at least %d = %d + 1\n",
+	   (polyDim + 1), polyDim);
+    puts("(nef.x -G requires POLY_Dmax >= dim(cone) = dim(support) + 1)");
+    exit(0);}
+}
+ 
+int  ReadCwsPp(CWS *_CW, PolyPointList *_P, int codim, int index)
+/*   _P is always an M-lattice polytope
+     codim = 1, index = 1: CY hypersurface 
+     codim > 1, index = 1: _P reflexive, CICY with codimension codim
+     codim = 1, index > 1: _P a Gorenstein polytope with index index, 
+                           determines a reflexive Gorenstein cone       */
 {    int i, j, FilterFlag=(inFILE==NULL); 
      int IN[AMBI_Dmax*(AMBI_Dmax+1)], S; 
      static int InputOK;
      _CW->nw=_CW->N=_CW->nz=0;
+     _CW->index = index;
      if(FilterFlag) inFILE=stdin; 
-     else if(inFILE==stdin)     {	
-       printf("Degrees and weights  `d1 w11 w12 ... d2 w21 w22 ...'\n  or `#");
-       printf("lines #colums' (= `PolyDim #Points' or `#Points PolyDim'):\n");}
+     else if(inFILE==stdin)     {
+       puts("Degrees and weights  `d1 w11 w12 ... d2 w21 w22 ...'");
+       puts("  or `#lines #columns' (= `PolyDim #Points' or `#Points PolyDim'):");}
      for(i=0;i<AMBI_Dmax*(AMBI_Dmax+1);i++)
      { char c;
        while(' '==(c=fgetc(inFILE ))); ungetc(c,inFILE);    /* read blanks */
@@ -118,11 +141,106 @@ int  Read_CWS_PP(CWS *_CW, PolyPointList *_P)
      if(i==1) { puts("Error in INPUT: need at least 2 numbers!"); exit(0);}
      if(i==2) 					      /* READ PolyPointList */
      {	int tr=0;    while('\n'!=fgetc(inFILE));      /* read to end of line */
+       if(IN[0]==IN[1]){ 
+	 puts("The number of points must be larger than the dimension!");
+	 exit(0);}
+       if(IN[0]>IN[1]) {tr=IN[0];IN[0]=IN[1];IN[1]=tr;} tr=!tr;
+	checkDimension(IN[0], codim, index);
+ 	if(IN[1]>POINT_Nmax) {puts("Please increase POINT_Nmax"); exit(0);}
+ 	if((inFILE==stdin)&&!FilterFlag)  
+	  printf("Type the %d coordinates as %s=%d lines with %s=%d columns:\n",
+		IN[0]*IN[1], tr ? "dim" : "#pts", tr ? IN[0] : IN[1],
+		tr ? "#pts" : "dim", tr ? IN[1] : IN[0]);
+	_P->n=IN[0]; _P->np=IN[1];
+	/* allow all numbers in one string (distributed over lines) or
+	 * matrix blocks with trailing comments	*/
+	if(tr) for(i=0;i<IN[0];i++) for(j=0;j<IN[1];j++)
+	  {   int X; fscanf(inFILE,"%d",&X); _P->x[j][i]=X;	}
+	else for(i=0;i<IN[1];i++) for(j=0;j<IN[0];j++)
+	  {   int X; fscanf(inFILE,"%d",&X); _P->x[i][j]=X;	}
+	/* Finish_Poly_Points(_P); */
+	while(fgetc(inFILE )-'\n') if(feof(inFILE)) return 0;/* read to EOL */
+     	if(FilterFlag) inFILE=NULL; return 1;
+     }  /* End of reading PolyPointList */
+     assert(i!=3);
+     S=IN[i-1]; 
+     for(j=0;j<i-1;j++) if((IN[j]==0)||(S<IN[j])) break; 
+     if(j==i-1)				 /* Single Weights with: "w1 ... d" */
+     {	_CW->nw=1; _CW->d[0]=S; _CW->N=i-1; 
+        assert(_CW->N <= POLY_Dmax+1);		      /* Increase POLY_Dmax */ 
+	for(j=0;j<_CW->N;j++) {_CW->W[0][j]=IN[j]; assert(IN[j]>0);} 
+	goto MAP;
+     }
+     _CW->d[0] = IN[0];                        /* ASSIGN (COMBINED) WEIGHTS */
+     S = IN[0] * index; 		     
+     for(j=1; j<i; j++)
+     {	if(S<IN[j]) 
+	{   if(S) { puts("Error in INPUT: degree vs. weights!"); exit(0);}
+	    else  
+	    {	_CW->d[++(_CW->nw)]=IN[j];
+	        S = IN[j] * index; 
+		if(1==_CW->nw) 
+		{   if(i%(1+_CW->N)) {puts("INPUT error: numbers?"); exit(0);}
+		}   if(j%(1+_CW->N)) {puts("INPUT error: degrees?"); exit(0);}
+	    }
+	}
+	else
+	{   _CW->W[_CW->nw][j - 1 - (1 + _CW->N)*_CW->nw] = IN[j]; S-=IN[j];
+	    if(0==_CW->nw) if(++(_CW->N)>AMBI_Dmax) 
+	    {   puts("Increase AMBI_Dmax!"); exit(0);
+	    }
+	}
+     }
+     ++(_CW->nw);
+     checkDimension(_CW->N - _CW->nw, codim, index);
+
+MAP: for(i=0;i<_CW->nw;i++)			/* check consistency of CWS */
+     {	Long sum = _CW->d[i] * index, *w = _CW->W[i];
+	for(j=0;j<_CW->N;j++) { assert(w[j]>=0); sum-=w[j]; }
+	if((sum)&&(index==1)){ 
+	  printf("Use poly.x -l for (single) WeightSystems with ");
+	  printf("d!=\\sum(w)\n(only Read_Weight makes the correct ");
+	  puts("PolyPointList in that case)"); exit(0);}
+     }
+     _CW->nz=0;
+     if(!Read_CWS_Zinfo(inFILE,_CW))			   /* read Z to EOL */
+     {	if(!InputOK) puts("-h gives you help\n"); return 0;}
+     Make_CWS_Points(_CW, _P);
+     if(FilterFlag) inFILE=NULL; return 1;
+}
+
+int  Read_CWS_PP(CWS *_CW, PolyPointList *_P){
+  return ReadCwsPp(_CW, _P, 1, 1);
+}
+
+int  Read_PP(PolyPointList *_P)
+{    int i, j, FilterFlag=(inFILE==NULL);
+     int IN[AMBI_Dmax*(AMBI_Dmax+1)];
+     static int InputOK;
+     /* _CW->nw=_CW->N=_CW->nz=0; */
+
+     if(FilterFlag) inFILE=stdin;
+     else if(inFILE==stdin)     {
+       printf("`#lines #columns' (= `PolyDim #Points' or `#Points PolyDim'):\n");
+     };
+     for(i=0;i<AMBI_Dmax*(AMBI_Dmax+1);i++)
+     { char c;
+       while(' '==(c=fgetc(inFILE ))); ungetc(c,inFILE);    /* read blanks */
+       if(IsNextDigit()) fscanf(inFILE,"%d",&IN[i]); else break;
+     }
+     if(i==0) { if(!InputOK){puts("-h gives you help\n"); exit(0);}
+	else return 0; } InputOK++;
+     if(i==1) { puts("Error in INPUT: need at least 2 numbers!"); exit(0);}
+     if(i==2) 					      /* READ PolyPointList */
+     {	int tr=0;    while('\n'!=fgetc(inFILE));      /* read to end of line */
+       if(IN[0]==IN[1]){ 
+	 puts("The number of points must be larger than the dimension!");
+	 exit(0);}
         if(IN[0]>IN[1]) {tr=IN[0];IN[0]=IN[1];IN[1]=tr;} tr=!tr;
 	if(IN[0]>POLY_Dmax) {puts("increase POLY_Dmax!"); exit(0);}
  	if(IN[1]>POINT_Nmax) {puts("increase POINT_Nmax!"); exit(0);}
- 	if((inFILE==stdin)&&!FilterFlag)  
-	  printf("Type the %d coordinates as %s=%d lines with %s=%d colums:\n",
+ 	if((inFILE==stdin)&&!FilterFlag)
+	  printf("Type the %d coordinates as %s=%d lines with %s=%d columns:\n",
 		IN[0]*IN[1], tr ? "dim" : "#pts", tr ? IN[0] : IN[1],
 		tr ? "#pts" : "dim", tr ? IN[1] : IN[0]);
 	_P->n=IN[0]; _P->np=IN[1];
@@ -135,28 +253,53 @@ int  Read_CWS_PP(CWS *_CW, PolyPointList *_P)
 	/* Finish_Poly_Points(_P); */
 	while(fgetc(inFILE )-'\n') if(feof(inFILE)) return 0;/* read to EOL */
      	if(FilterFlag) inFILE=NULL; return 1;
-     }  assert(i!=3);
-     S=IN[i-1]; for(j=0;j<i-1;j++) if(S<IN[j]) break; 
+     }
+     if(i>2){puts("Error: expected input format is matrix of polytope points!");exit(0);}
+     if(FilterFlag) inFILE=NULL; return 1;
+}
+
+int  Read_CWS(CWS *_CW, PolyPointList *_P)
+{    int i, j, FilterFlag=(inFILE==NULL);
+     int IN[AMBI_Dmax*(AMBI_Dmax+1)], S;
+     static int InputOK;
+     _CW->nw=_CW->N=_CW->nz=0;
+     _CW->index = 1;
+
+     if(FilterFlag) inFILE=stdin;
+     else if(inFILE==stdin)     {
+       printf("Degrees and weights  `d1 w11 w12 ... d2 w21 w22 ...':\n");
+     };
+     for(i=0;i<AMBI_Dmax*(AMBI_Dmax+1);i++)
+     { char c;
+       while(' '==(c=fgetc(inFILE ))); ungetc(c,inFILE);    /* read blanks */
+       if(IsNextDigit()) fscanf(inFILE,"%d",&IN[i]); else break;
+     }
+     if(i==0) { if(!InputOK){puts("-h gives you help\n"); exit(0);}
+	else return 0; } InputOK++;
+     if(i==1) { puts("Error in INPUT: need at least 2 numbers!"); exit(0);}
+     if(i==2) {puts("Error: expected input format is CWS!"); exit(0);}
+     assert(i!=3);
+     S=IN[i-1]; for(j=0;j<i-1;j++) if(S<IN[j]) break;
      if(j==i-1)				 /* Single Weights with: "w1 ... d" */
-     {	_CW->nw=1; _CW->d[0]=S; _CW->N=i-1; 
-        assert(_CW->N <= POLY_Dmax+1);		      /* Increase POLY_Dmax */ 
-	for(j=0;j<_CW->N;j++) {_CW->W[0][j]=IN[j]; assert(IN[j]>0);} 
+     {	_CW->nw=1; _CW->d[0]=S; _CW->N=i-1;
+        assert(_CW->N <= POLY_Dmax+1);		      /* Increase POLY_Dmax */
+	for(j=0;j<_CW->N;j++) {_CW->W[0][j]=IN[j]; assert(IN[j]>0);}
 	goto MAP;
      }
      S=_CW->d[_CW->nw]=IN[0]; 			          /* ASSIGN WEIGHTS */
      for(j=1; j<i; j++)
-     {	if(S<IN[j]) 
+     {	if(S<IN[j])
 	{   if(S) { puts("Error in INPUT: degree vs. weights!"); exit(0);}
-	    else  
+	    else
 	    {	S=_CW->d[++(_CW->nw)]=IN[j];
-		if(1==_CW->nw) 
+		if(1==_CW->nw)
 		{   if(i%(1+_CW->N)) {puts("INPUT error: numbers?"); exit(0);}
 		}   if(j%(1+_CW->N)) {puts("INPUT error: degrees?"); exit(0);}
 	    }
 	}
 	else
 	{   _CW->W[_CW->nw][j - 1 - (1 + _CW->N)*_CW->nw] = IN[j]; S-=IN[j];
-	    if(0==_CW->nw) if(++(_CW->N)>AMBI_Dmax) 
+	    if(0==_CW->nw) if(++(_CW->N)>AMBI_Dmax)
 	    {   puts("Increase AMBI_Dmax!"); exit(0);
 	    }
 	}
@@ -171,7 +314,7 @@ MAP: for(i=0;i<_CW->nw;i++)			/* check consistency of CWS */
 	for(j=0;j<_CW->N;j++) { assert(w[j]>=0); sum-=w[j]; }
 	if(sum){ printf("Use poly.x -w for (single) WeightSystems with ");
 		printf("d!=\\sum(w)\n(only Read_Weight makes the correct ");
-		puts("PolyPointList in that case"); exit(0);}
+		puts("PolyPointList in that case)"); exit(0);}
      }
      _CW->nz=0;
      if(!Read_CWS_Zinfo(inFILE,_CW))			   /* read Z to EOL */
@@ -326,7 +469,8 @@ void Reduce_PPL_2_Sublat(PolyPointList *P,int *nm,Long *M,Long G[][POLY_Dmax])
 Long PD_Floor(Long N,Long D)	/*  assuming PosDenom  D>0:  F <= N/D < F+1  */
 {    Long F=N/D; return (F*D>N) ? F-1 : F; 
 }
-void Make_CWS_Points(CWS *Cin, PolyPointList *_P)
+
+void Old_Make_CWS_Points(CWS *Cin, PolyPointList *_P)
 {    int i, j, Amin[POLY_Dmax+1]; Long *x=_P->x[_P->np=0], xmin[POLY_Dmax], 
 	xmax[POLY_Dmax], Xmax[AMBI_Dmax], xaux[POLY_Dmax], L, R; CWS *_C=Cin;
      CWLatticeBasis B; Long G[POLY_Dmax][POLY_Dmax],M[POLY_Dmax];int m=Cin->nz;
@@ -399,6 +543,119 @@ void Make_CWS_Points(CWS *Cin, PolyPointList *_P)
      }	if(m)CWS_2_SublatZ(_C,&B,&m,M,G);if(m)Reduce_PPL_2_Sublat(_P,&m,M,G);
 }
 
+int Compute_X0(int N, CWS *_C, Long *X0){
+  /* finds a first X = X0 such that sum w_i X_i = d for every weight w in _C,
+     this becomes the origin after the change of coordinates from X to x */
+  int j;
+  Long Xmax=0; 
+  if (!N){
+    for(j=0;j<_C->nw;j++) 
+      if(_C->W[j][0]){
+	if (_C->d[j]%_C->W[j][N]) return 0;
+	if(Xmax) {if (Xmax != _C->d[j]/_C->W[j][N]) return 0;}
+	else Xmax = _C->d[j]/_C->W[j][N];   }
+      else if(_C->d[j]) return 0;
+    X0[0] = Xmax;
+    return 1;}
+  for(j=0;j<_C->nw;j++) if(_C->W[j][N]!=0){
+      Long L=_C->d[j]/_C->W[j][N]; 
+      if(Xmax) {if(L<Xmax) Xmax = L;}
+      else Xmax = L;   }
+  for (X0[N]=0; X0[N] <= Xmax; X0[N]++){
+    if (Compute_X0(N-1, _C, X0)){
+      for(j=0; j<_C->nw; j++) _C->d[j] += X0[N] * _C->W[j][N]; /* reset Cin*/
+      return 1;}
+    for(j=0; j<_C->nw; j++) _C->d[j] -= _C->W[j][N];}
+  for(j=0; j<_C->nw; j++) _C->d[j] += (Xmax + 1) * _C->W[j][N]; /* reset Cin */
+  return 0;
+}
+
+void Make_CWS_Points(CWS *Cin, PolyPointList *_P)
+{    int i, j, Amin[POLY_Dmax+1], m=Cin->nz; 
+     Long *x=_P->x[_P->np=0], xmin[POLY_Dmax], 
+       xmax[POLY_Dmax], Xmax[AMBI_Dmax], X0[AMBI_Dmax], xaux[POLY_Dmax], L, R; 
+     CWS *_C=Cin;
+     CWLatticeBasis B; Long G[POLY_Dmax][POLY_Dmax],M[POLY_Dmax];
+#ifndef NO_COORD_IMPROVEMENT		/* ==== Perm Coord Improvement ==== */
+     int pi[AMBI_Dmax]; CWS Caux; _C=&Caux; CWS_to_PermCWS(Cin,_C, pi);
+#endif				       /* = End of Perm Coord Improvement = */
+     Make_CWS_Basis(_C, &B);	  		 /* make `triangular' Basis */
+#ifndef NO_COORD_IMPROVEMENT		/* ==== Perm Coord Improvement ==== */
+     assert(_C->N==_C->B.ne);
+     for(i=0;i<Cin->N;i++) Cin->B.e[i]=_C->B.e[pi[i]]; Cin->B.ne=_C->N;
+#endif				       /* = End of Perm Coord Improvement = */
+     if (Cin->index == 1) for (i=0; i< Cin->N; i++) X0[i] = 1;
+     else if(!Compute_X0(Cin->N - 1, Cin, X0)){_P->n=0; puts("no X0!");return;}
+     /* X0 is the reference point in X-space that is transformed to the 
+	origin of x-space for _P    */
+     /*printf("\nX0: ");for (i=0;i<Cin->N;i++) printf("%ld ", X0[i]);puts("");
+     for(i=0;i<Cin->nw;i++)     {	
+       fprintf(outFILE,"%d ",(int) Cin->d[i]);
+       for(j=0;j<Cin->N;j++) fprintf(outFILE,"%d ",(int) Cin->W[i][j]);
+       if(i+1<Cin->nw) fprintf(outFILE," ");     }*/
+     i=_P->n=B.n; Amin[0]=0; Amin[B.n]=j=B.N;	      /* inversion structure */
+     while(--i) {while(!B.x[i-1][--j]); Amin[i]=++j;}
+     for(i=0;i<B.N;i++) 				     /* compute Xmax */
+     {	Xmax[i]=0; for(j=0;j<_C->nw;j++) if(_C->W[j][i])
+	{   L=_C->d[j]/_C->W[j][i]; 
+	    if(Xmax[i]) {if(L<Xmax[i]) Xmax[i]=L;}
+	    else Xmax[i]=L;
+	}
+     }
+     j=B.n-1; i=Amin[j+1]-1; R=B.x[j][i];     /* compute xmin[j] and xmax[j] */
+     xmin[j] = -PD_Floor(X0[i],R); 
+     xmax[j] = PD_Floor(Xmax[i]-X0[i],R);     /* since R > 0 */
+/** /printf("R=%2d:  %2d <= x[%d=&%d] <= %2d\n",R,xmin[j],j,i,xmax[j]);/ **/
+     while((i--)>Amin[j]) 	/* if(R=B.x[B.n-1][i]):  R!=0  => new limits */
+     {	Long Low=-X0[i], Upp=Low+Xmax[i]; 		  R=B.x[B.n-1][i];
+	if(R>0) {if(xmax[j]>(L=PD_Floor(Upp,R)))   xmax[j]=L;
+		 if(xmin[j]<(L=-PD_Floor(-Low,R))) xmin[j]=L;}
+	else	{if(xmax[j]>(L=PD_Floor(-Low,-R))) xmax[j]=L;
+		 if(xmin[j]<(L=-PD_Floor(Upp,-R))) xmin[j]=L;}
+/** /printf("R=%2d:  %2d <= x[%d=&%d] <= %2d\n",R,xmin[j],j,i,xmax[j]);/ **/
+     }				   /* this completes the limits for x[B.n-1] */
+     x[j]=xmin[j]; 
+     while(j<B.n)
+     {	int k; if(x[j]>xmax[j]) {if(B.n==(++j)) break; else x[j]++;}
+	else		/* compute limits[j-1] and initialize x[j-1] */
+	{   Long Upp=Xmax[i=Amin[j--]-1], Low=-X0[i]; int RangeFlag=0;
+	    for(k=j+1;k<B.n;k++) Low-=x[k]*B.x[k][i];	   /* compute offset */
+	    Upp+=Low; R=B.x[j][i]; 
+	    xmin[j]=-PD_Floor(-Low,R); xmax[j]=PD_Floor(Upp,R);
+/** /printf("R=%2d:  %2d <= x[%d=&%d] <= %2d\n",R,xmin[j],j,i,xmax[j]);/ **/
+	    while((i--)>Amin[j]) if((R=B.x[j][i]))    /* R!=0  => new limits */
+     	    {	Low=-X0[i]; Upp=Xmax[i];
+		for(k=j+1;k<B.n;k++) Low-=x[k]*B.x[k][i];
+		Upp+=Low; R=B.x[j][i]; 
+		if(R>0) {if(xmax[j]>(L=PD_Floor(Upp,R)))   xmax[j]=L;
+		 	 if(xmin[j]<(L=-PD_Floor(-Low,R))) xmin[j]=L;}
+		else	{if(xmax[j]>(L=PD_Floor(-Low,-R))) xmax[j]=L;
+			 if(xmin[j]<(L=-PD_Floor(Upp,-R))) xmin[j]=L;}
+/** /printf("R=%2d:  %2d <= x[%d=&%d] <= %2d\n",R,xmin[j],j,i,xmax[j]);/ **/
+	    }			     	/* completes limits for x[] except */
+	    else			/*   when R=0 and X out of Range:  */
+	    {	Long X=1; for(k=j+1;k<B.n;k++) X+=x[k]*B.x[k][i];
+		if((X<0)||(X>Xmax[i])) RangeFlag=1;
+	    }
+	    if(RangeFlag) ++x[++j]; else x[j]=xmin[j];            
+	    if(j==0)
+	    {   while(x[0]<=xmax[0]) 
+	    	{   Long *y; 
+		    if((++_P->np) < POINT_Nmax) 
+		    {	y=(_P->x[_P->np]); for(k=0;k<B.n;k++) y[k]=x[k]; 
+		    }
+		    else if(_P->np == POINT_Nmax)
+		    {	y=xaux; for(k=0;k<B.n;k++) y[k]=x[k];
+		    }
+		    else {puts("Increase POINT_Nmax");exit(0);}
+		    x=y; ++x[0];
+	    	}
+	        x[j=1]++;
+	    }
+	}
+     }	if(m)CWS_2_SublatZ(_C,&B,&m,M,G);if(m)Reduce_PPL_2_Sublat(_P,&m,M,G);
+}
+
 /*  ==========    Coordinate improvement via CWS-Permutations   ==========  */
 
 typedef struct {Long s, g, *N, **G; int *p;}		Tri_GLZ_MPaux;  
