@@ -38,33 +38,12 @@ be considerably slowed down.
 #define   		FACE_Nmax	824	/* max number of faces      */
 #define	                SYM_Nmax	1200
 
-#elif   (POLY_Dmax == 5)
-#define                 POINT_Nmax      355785  /* max number of points     */
-#define                 VERT_Nmax       64      /* max number of vertices   */
-#define                 FACE_Nmax       5000    /* max number of faces      */
-#define                 SYM_Nmax        5000
-#define                 EQUA_Nmax       128
-
-#elif   (POLY_Dmax == 6)
-#define                 POINT_Nmax      500000  /* max number of points     */
-#define                 VERT_Nmax       64      /* max number of vertices   */
-#define                 FACE_Nmax       5000    /* max number of faces      */
-#define                 SYM_Nmax        46080
-#define                 EQUA_Nmax       512
-
-#elif   (POLY_Dmax < 12)
-#define                 POINT_Nmax      2000000 /* max number of points     */
-#define                 VERT_Nmax       128     /* max number of vertices   */
-#define                 FACE_Nmax       5000    /* max number of faces      */
-#define                 SYM_Nmax        46080   /* symmetry (P_1)^6: 2^6*6! */
-#define                 EQUA_Nmax       1280
-
 #else
-#define   		POINT_Nmax	640     /* max number of points	    */
+#define   		POINT_Nmax	2000000 
 #define   		VERT_Nmax	64	/* !! use optimal value !!  */
 #define   		FACE_Nmax	10000	/* max number of faces      */
-#define                 SYM_Nmax        9999
-#define			EQUA_Nmax	55000   /* up to 20000 without alloc */
+#define                 SYM_Nmax        46080   /* symmetry (P_1)^6: 2^6*6! */
+#define			EQUA_Nmax	1280    /* up to 20000 without alloc */
 #endif
 
 #ifndef			EQUA_Nmax			/* default setting */
@@ -93,6 +72,27 @@ by an embeddeding in an ambient space of dimension (Poly-dim + number of
 weight systems). AMBI_Dmax is the maximal dimension of this ambient space.
 */
 
+
+#define                 FIB_Nmax	3000 /*NOW: 27/5/11 default setting*/
+/*
+Given a polytope P* it is possible to analyze the IP simplices among its 
+points. These simplices are given in terms of weight relations among points 
+of P*. FIB_Nmax is the maximal number of allowed relations.
+*/
+
+
+#define  CD2F_Nmax               FACE_Nmax
+/*
+Max number of codimension 2 faces.
+*/
+
+
+#define GL_Long		Long
+/*
+Uses W_to_GLZ like in Rat.c
+*/
+
+
 extern FILE *inFILE, *outFILE;
 /*
 Ascii-files for input and output. If not given in the parameter list they
@@ -127,9 +127,8 @@ typedef struct {int ne; Equation e[EQUA_Nmax];}		     EqList;
 A list of equations; EL.ne is the number of equations in the list.
 */
 
-typedef struct {Long W[AMBI_Dmax][AMBI_Dmax], d[AMBI_Dmax]; int nw, N,
-		z[POLY_Dmax][AMBI_Dmax], m[POLY_Dmax], nz;
-		EqList B;}				     CWS;
+typedef struct {EqList B; Long W[AMBI_Dmax][AMBI_Dmax], d[AMBI_Dmax]; 
+  int nw, N, z[POLY_Dmax][AMBI_Dmax], m[POLY_Dmax], nz, index;}       CWS;
 /*
 Combined weight system: W[i][j] and d[i] are the j'th weight and the "degree"
 of the i'th weight system, respectively; nw is the number of weight systems,
@@ -158,6 +157,19 @@ mp, mv, np, nv denote the numbers of points/vertices in the M and N lattices,
 repectively.
 */
 
+typedef struct {
+	Long W[FIB_Nmax][VERT_Nmax]; 
+	int nw, PS, ZS, nv, f[VERT_Nmax],r[VERT_Nmax],nf,nz[FIB_Nmax], n0[FIB_Nmax],
+         Z[FIB_Nmax][VERT_Nmax], M[FIB_Nmax];
+	GL_Long G[VERT_Nmax][POLY_Dmax][POLY_Dmax];
+	PolyPointList *P;
+} FibW;
+/*
+This list is an extension of the PolyPointList with the combined weight system.
+W[i][j] is the j'th weight; nw is the number of weight systems.
+*/
+
+
 
 /*  ==========         I/O functions (from Coord.c)		==========  */
 
@@ -169,12 +181,22 @@ C->nw is set to 0 to indicate that no weight has been read.
 CWS-input consists of a single line of the form
 d1 w11 w12 ... d2 w21 w22 ...,
 whereas PolyPointList-input begins with a line
-#colums #lines
+#columns #lines
 followed by #lines further lines. It reads P->x as a matrix such that
-either P->n = #colums and P->np = #lines or vice versa (the result is
+either P->n = #columns and P->np = #lines or vice versa (the result is
 unique because of P->np > P->n).
 */
 
+int  Read_CWS(CWS *_CW, PolyPointList *_P);
+/*
+ Reads CWS input *C, the PolyPointList *P determined by *C is calculated.
+*/
+
+int  Read_PP(PolyPointList *_P);
+/*
+Reads the PolyPointList input *P
+*/
+
 void Print_PPL(PolyPointList *P, const char *comment);
 void Print_VL(PolyPointList *P, VertexNumList *V, const char *comment);
 void Print_EL(EqList *EL, int *n, int suppress_c, const char *comment);
@@ -182,7 +204,7 @@ void Print_Matrix(Long Matrix[][VERT_Nmax], int n_lines, int n_columns,
 		  const char *comment);
 /*
 Each of these routines prints a matrix in the format
-#colums #lines  *comment
+#columns #lines  *comment
 line_0
 ...
 line_{#lines-1}.
@@ -224,7 +246,7 @@ As an example, the complete output for the quintic threefold reads
 
 int  Make_Poly_Sym_NF(PolyPointList *P, VertexNumList *VNL, EqList *EL,
 		      int *SymNum, int V_perm[][VERT_Nmax],
-		      Long NF[POLY_Dmax][VERT_Nmax], int traced);
+		      Long NF[POLY_Dmax][VERT_Nmax], int t, int S, int N);
 /*
 Given *P, *VNL and *EL, the following objects are determined:
 the number *SymNum of GL(n,Z)-symmetries of the polytope,
@@ -232,7 +254,8 @@ the *SymNum vertex permutations V_perm realising these symmetries,
 the normal form coordinates NF of the vertices,
 the number of symmetries of the vertex pairing matrix
     (this number is the return value of Make_Poly_Sym_NF).
-If `traced' is non-zero, extra information is displayed.
+If t/S/N are non-zero, the output of the corresponding options of poly.x
+is displayed.
 */
 
 void IP_Simplex_Decomp(Long CM[][POLY_Dmax], int p, int d,
@@ -249,7 +272,7 @@ coefficients.
 
 void IP_Simplices(PolyPointList *P, int nv, int PS, int VS, int CD);
 /*
-Realises the -P,-V,-Z, and fibration options of poly (the results of this
+Realizes the -P,-V,-Z, and fibration options of poly (the results of this
 routine are displayed as output; *P is not modified).
 */
 
@@ -274,6 +297,61 @@ A coordinate change is performed that makes the matrix P->x upper triangular,
 with minimal entries above the diagonal.
 */
 
+void Make_ANF(PolyPointList *P, VertexNumList *V, EqList*E, 
+	      Long ANF[][VERT_Nmax]);
+/*
+Given *P, *V and *E, the affine normal form ANF (i.e., a normal form
+that also works for non-reflexive polytopes), is computed.
+*/
+
+int SimpUnimod(PolyPointList *P, VertexNumList *V, EqList *E, int vol);
+/*
+If vol is 0, the return value is 1 if all facets are simplicial, 0 otherwise
+If vol is not 0, the return value is 1 if all facets are unimoular
+(i.e. of volume 1) and 0 otherwise.
+*/
+
+int ConifoldSing(PolyPointList *P, VertexNumList *V, EqList *E,
+		 PolyPointList *dP, EqList *dE, int CYorFANO);
+/*
+Realizes the -C1 or -C2 options of poly for CYorFANO being 1 or 2, respectively.
+*/
+
+int  Fano5d(PolyPointList *, VertexNumList *, EqList *);
+/*
+Realizes the -U5 option of poly.
+*/
+
+void Einstein_Metric(CWS *CW,PolyPointList *P,VertexNumList *V,EqList *E);
+/*
+Realizes the -E option of poly.
+*/
+
+int  Divisibility_Index(PolyPointList *P, VertexNumList *V);
+/*
+Returns the largest integer g for which *P is a g-fold multiple of some
+other polytope.
+*/
+
+Long LatVol_Barycent(PolyPointList *P, VertexNumList *V, Long *B, Long *N);
+/*
+Given *P and *V, the coordinates of the barycenter of *P are computed (with 
+the i'th coordinate as B[i] / *N) and the lattice volume of *P is returned.
+*/
+
+void IPs_degD(PolyPointList *P, VertexNumList *V, EqList *E, int l);
+/*
+ *P is interpreted as the origin and the first level of a Gorenstein cone. 
+The points of the cone up to level l are computed and displayed together with 
+information on the type of face of the cone they represent (option -B# of poly).
+*/
+
+void Make_Facet(PolyPointList *P, VertexNumList *V, EqList *E, int e, 
+		Long vertices_of_facet[POLY_Dmax][VERT_Nmax], int *nv_of_facet);
+/*
+The e'th facet of *P is determined as a (P->n-1)-dimensional polytope:
+*nv_of_facet vertices represented by vertices_of_facet.
+*/
 
 /*  ==========     General purpose functions from Vertex.c   	==========  */
 
