@@ -932,7 +932,7 @@ void AuxVnf2ucNF(Long nf[POLY_Dmax][VERT_Nmax], int *d, int *v,
 }
 void UCnf2vNF(int *d, int *v, int *nuc, unsigned char *uc,  	      /* IN */
 	      Long NF[POLY_Dmax][VERT_Nmax], int *MS)		     /* OUT */
-{    int i, j, off, base, one;
+{    int i, j, off=0, base, one;
      UXLong X; X.n=*nuc/2; if(*nuc % 2) X.x[X.n++]=uc[*nuc-1];
      for(i=0;i<*nuc/2;i++) X.x[i]=uc[2*i]+UCM*uc[2*i+1]; 	i=UNIT_OFF; 
      BaseGetInt(&i,MS,&X); one = (*MS > 3);
@@ -1108,7 +1108,7 @@ void Add_Polya_2_Polyi(char *polyi,char *polya,char *polyo)
      FInfoList FIi, FIa, FIo;	  Along Ipos, Apos, HIpos, HApos;
      unsigned char ucI[NUC_Nmax], ucA[NUC_Nmax], *ucSL, *uc; int SLp[SL_Nmax];
      int IslNF, IslSM, IslNM, vI, nuI, i, d; unsigned Ili; Along IslNB; 
-     int AslNF, AslSM, AslNM, vA, nuA, a, j; unsigned Ali; Along AslNB; 
+     int AslNF, AslSM, AslNM, vA=0, nuA=0, a, j; unsigned Ali; Along AslNB; 
      int slNF=0,slSM=0,slNM=0, slNB=0, slNP=0, Oli=0, v, nu, AmI=0, ms, tnb=0; 
      Init_FInfoList(&FIi); Init_FInfoList(&FIa);        if(!*polyi||!*polyo) {
        puts("With -pa you require -pi and -po or -di and -do");exit(0);}
