@@ -1495,18 +1495,6 @@ void Test_Hodge_file(char *filename, PolyPointList *_P){
 	   nuc,mirror,mv,mp,nv,np,MS); }
   fclose(Ft);
 }
-#else
-void DB_to_Hodge(char *dbin, char *dbout, int vfrom, int vto, 
-		 PolyPointList *_P){
-  puts("For using Hodge-DB-routines set POLY_Dmax=4!");exit(0);}
-void Sort_Hodge(char *dbaux, char *dbout){
-  puts("For using Hodge-DB-routines set POLY_Dmax=4!");exit(0);}
-void Test_Hodge_file(char *filename, PolyPointList *P){
-  puts("For using Hodge-DB-routines set POLY_Dmax=4!");exit(0);}
-void Test_Hodge_db(char *dbname){
-  puts("For using Hodge-DB-routines set POLY_Dmax=4!");exit(0);}
-void Extract_from_Hodge_db(char *dname, char *x_string, PolyPointList *P){
-  puts("For using Hodge-DB-routines set POLY_Dmax=4!");exit(0);}
 #endif
 
 /*  =====================================================================  */
