@@ -2,7 +2,8 @@
 #include "Rat.h"
 
 #define MAX_BAD_EQ	(POLY_Dmax>5)	/* previously 6; needed for nef !? */
-#define SHOW_NEW_CEq    (POLY_Dmax>12)
+#define SHOW_NEW_CEq    0               /* (POLY_Dmax>12) 
+					   tracks polytope analysis  */
 #ifndef	CEQ_Nmax	
 #define CEQ_Nmax        EQUA_Nmax
 #endif
@@ -116,7 +117,8 @@ void Make_Incidence(PolyPointList *_P, VertexNumList *_V, EqList *_E,
      assert(_E->ne<=VERT_Nmax);
      _I->nf[M]=_E->ne; 
      for(i=0;i<_E->ne;i++)_I->v[M][i]=Eq_To_INCI(&_E->e[i],_P,_V); /* init .v*/
-     _I->f[M][--i]=INCI_1(); 
+     assert(i>0);
+     _I->f[M][--i]=INCI_1();
      while(i--) _I->f[M][i]=INCI_PN(_I->f[M][i+1],1);    	  /* init .f */
      while((D=d--))
      {	int *n=&_I->nf[d];  *n=0;			       /* #(d-faces) */
